/* ----------------------------------------------------------------------
   (C) Copyright Marxmeier Software AG 2006,2007,2008

   Project:  Eloquence database
   File:     fwutil.h
   Function: fw util library abstraction
   $Id: fwutil.h,v 27.1 2011/02/10 13:57:24 mike Exp $

   Edit History:
   30.05.2006, mike - creation
   29.03.2007, rhg  - rev. B.07.10.05: session info api
   12.03.2008, rhg  - rev. B.07.10.06: memo api

---------------------------------------------------------------------- */

#if !defined(FWUTIL_H_INCLUDED)
#define FWUTIL_H_INCLUDED

/* ------------------------------------------------------------------------
   includes
   ------------------------------------------------------------------------ */

#include <time.h>


/* ------------------------------------------------------------------------
   function prototypes
   ------------------------------------------------------------------------ */

/* print fwutil version information */
extern void Fwu_version(void);

/* print fwutil status file */
extern int Fwu_status(const char *status_file, int verbose);

/* sync only then exit */
extern void Fwu_synconly(int mode);

/* set verbose mode */
extern void Fwu_verbose(int mode);

/* callback */
enum Fwu_OP {
   Fwu_OP_UPDATE = 1,
   Fwu_OP_PUT    = 3,
   Fwu_OP_DELETE = 5
};

typedef int (*Fwu_callback_t)(int db_node_id, const char *db_name,
                   const char *set_name, time_t timestamp,
		   enum Fwu_OP op, unsigned int recno, int recsz,
                   const void *bi_rec, const void *ai_rec);

/* memo callback */
enum Fwu_MEMO {
   Fwu_MEMO_DBBEGIN = 0x62, /* 'b'  98d */
   Fwu_MEMO_DBEND   = 0x65, /* 'e' 101d */
   Fwu_MEMO_DBMEMO  = 0x6d  /* 'm' 109d */
};

typedef int (*Fwu_memo_callback_t)(time_t timestamp, int mode,
                  const void *data, int data_sz);

extern void Fwu_set_memo_callback(Fwu_memo_callback_t cb);

/* process */
extern int Fwu_process(Fwu_callback_t cb, const char *cfg_file,
             const char *status_file,
	     const char *fwr_start);

extern int Fwu_get_item(const char **i_name, int *i_type,
	                int *i_size, int *i_count, unsigned int *i_fmt);

extern int Fwu_get_session_id(void);
extern int Fwu_get_session_elements(void);
extern const char *Fwu_get_session_data(int element_idx);
extern const char *Fwu_get_session_entry(const char *key);
extern const void *Fwu_get_memo(int mode, time_t *timestamp, int *data_sz);

extern unsigned int Fwu_CurrentLogGeneration(void);
extern unsigned int Fwu_CurrentLogSequence(void);
extern unsigned int Fwu_CurrentTagSequence(void);

/* conversion/formatting */
extern int Fwu_fmt_signed_int(void *buf_p, int buf_sz, 
                              const void *v_p, int v_len);
extern int Fwu_fmt_unsigned_int(void *buf_p, int buf_sz, 
                                const void *v_p, int v_len);
extern int Fwu_fmt_ieee_float(void *buf_p, int buf_sz, 
                              const void *v_p, int v_len);
extern int Fwu_fmt_packed(void *buf_p, int buf_sz, 
                          const void *v_p, int v_len);
extern int Fwu_fmt_zoned(void *buf_p, int buf_sz, 
                         const void *v_p, int v_len);

#endif /* FWUTIL_H_INCLUDED */
